package com.bsz.school_send_back_end.service.impl;


import com.alipay.api.AlipayApiException;
import com.alipay.api.domain.AlipayTradeCreateModel;
import com.alipay.api.request.AlipayTradeCreateRequest;
import com.alipay.api.response.AlipayTradeCreateResponse;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.bsz.school_send_back_end.common.ErrorCode;
import com.bsz.school_send_back_end.config.AlipayClients;
import com.bsz.school_send_back_end.exception.BusinessException;
import com.bsz.school_send_back_end.exception.ThrowUtils;
import com.bsz.school_send_back_end.model.domain.OrderDetails;
import com.bsz.school_send_back_end.model.domain.Orders;
import com.bsz.school_send_back_end.service.AliPayService;
import com.bsz.school_send_back_end.service.OrderDetailsService;
import com.bsz.school_send_back_end.service.OrdersService;
import com.bsz.school_send_back_end.utils.UniqueNumberGenerator;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class AlipayServiceImpl implements AliPayService {

    @Resource
    private OrdersService ordersService;

    @Resource
    private OrderDetailsService orderDetailsService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public synchronized String createPayment(String orderId, String miniOpenId, BigDecimal amount, Long businessId) {
        //创建客户
        AlipayClients clients = new AlipayClients();
        //AlipayClient client = clients.alipayClient();
        //创建交易请求
        AlipayTradeCreateRequest aliRequest = new AlipayTradeCreateRequest();
        //创建实体类
        AlipayTradeCreateModel model = new AlipayTradeCreateModel();

        //请求参数 沙箱内部参数

        String outTradeNo = UniqueNumberGenerator.generateNumber(businessId);

        Orders order = ordersService.getById(orderId);
        if (order == null) {
            log.error("订单不存在");
            throw new BusinessException(ErrorCode.NOT_FOUND_ERROR, "订单不存在，订单号：" + outTradeNo);
        }

        LambdaQueryWrapper<OrderDetails> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(OrderDetails::getOrderId, orderId);
        List<OrderDetails> orderDetailsList = orderDetailsService.list(wrapper);
        String subject = orderDetailsList.stream()
                .map(OrderDetails::getAttributeNames)
                .collect(Collectors.joining(", "));
        //给支付宝的订单号设置成取餐码
        order.setPickupCode(outTradeNo);
        boolean update = ordersService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "修改订单状态失败");

        model.setOutTradeNo(outTradeNo);
        model.setTotalAmount(String.valueOf(amount));
        model.setSubject(subject);
        model.setProductCode("JSAPI_PAY");
        model.setSellerId("2088721037756350");
        model.setBuyerId("2088722037756360");
        model.setOpAppId("2021004151684053");

        ///回调地址
        aliRequest.setNotifyUrl("http://39.101.78.35:6448/api/Alipay/notifyUrl");
        aliRequest.setBizModel(model);
        String trade_no = null;
        try {
            AlipayTradeCreateResponse response = clients.alipayClient().execute(aliRequest);
            trade_no = response.getTradeNo();
            if (trade_no == null) {
                throw new BusinessException(ErrorCode.NULL_ERROR, "未获取到trade_no");
            }
        } catch (AlipayApiException e) {
            e.printStackTrace();
        }
        return trade_no;
    }
}
