package com.bsz.school_send_back_end.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bsz.school_send_back_end.mapper.UserRatingMapper;
import com.bsz.school_send_back_end.model.domain.BusinessLevel;
import com.bsz.school_send_back_end.service.BusinessLevelService;
import com.bsz.school_send_back_end.mapper.BusinessLevelMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
* @author pc
* @description 针对表【business_level】的数据库操作Service实现
* @createDate 2024-11-22 14:02:11
*/
@Service
@Slf4j
public class BusinessLevelServiceImpl extends ServiceImpl<BusinessLevelMapper, BusinessLevel>
    implements BusinessLevelService{

    @Resource
    private BusinessLevelMapper businessLevelMapper;

    @Resource
    private UserRatingMapper userRatingMapper;

    @Override
    public void updateBusinessLevel(Long businessId) {
        Double avg = userRatingMapper.findAvgByBusinessId(businessId);
        if (avg == null) {
            avg = 1.0;
            log.info("warn: avg没有值，将赋初始值1：" + avg);
            log.info("当前商家id：" + businessId);
        }
        int level = calculateLevel(avg);
        BusinessLevel businessLevel = businessLevelMapper.selectOne(Wrappers.<BusinessLevel>lambdaQuery()
                                                            .eq(BusinessLevel::getBusinessId, businessId));

        if (businessLevel == null) {
            businessLevel = new BusinessLevel();
            businessLevel.setBusinessId(businessId);
            businessLevel.setLevel(1);
            businessLevel.setAverageScore(BigDecimal.valueOf(1));
        }
        businessLevel.setAverageScore(BigDecimal.valueOf(avg));
        businessLevel.setLevel(level);

        this.saveOrUpdate(businessLevel);
    }

    @Override
    public List<BusinessLevel> findAllOrdered() {
        List<BusinessLevel> list = businessLevelMapper.selectList(Wrappers.<BusinessLevel>lambdaQuery()
                .orderByDesc(BusinessLevel::getLevel,BusinessLevel::getAverageScore)
                .orderByAsc(BusinessLevel::getCreateTime));
        return list;
    }


    private int calculateLevel(Double averageScore) {
        if (averageScore >= 4.5) {
            return 5;
        } else if (averageScore >= 4.0) {
            return 4;
        } else if (averageScore >= 3.0) {
            return 3;
        } else if (averageScore >= 2.0) {
            return 2;
        } else {
            return 1;
        }
    }
}




